/*
 * active port forwarder - software for secure forwarding
 * Copyright (C) 2003,2004,2005 jeremian <jeremian [at] poczta.fm>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <config.h>

#include "server_find.h"

int
find_client(ServerRealm* ptr, char mode, int usrclipair)
{
  int i;
  switch(mode) {
    case 1: { /* fill first client before go to next */
              for (i = 0; i < ServerRealm_get_clientsLimit(ptr); ++i) {
                if ((ConnectClient_get_state(ServerRealm_get_clientsTable(ptr)[i]) ==
                      CONNECTCLIENT_STATE_ACCEPTED) &&
                    (ConnectClient_get_usrCliPair(ServerRealm_get_clientsTable(ptr)[i]) == usrclipair)) {
                  if (ConnectClient_get_connected(ServerRealm_get_clientsTable(ptr)[i]) <
                      ConnectClient_get_limit(ServerRealm_get_clientsTable(ptr)[i])) {
                    return i;
                  }
                }
              }
              break;
            }
    default: {
               return 0;
             }
  }
  return 0;
}

int
find_usernum(ConnectClient* ptr, int usernum)
{
  int i;
  for (i = 0; i < ConnectClient_get_limit(ptr); ++i) {
    if (ConnectClient_get_users(ptr)[i] == -1) {
      ConnectClient_get_users(ptr)[i] = usernum;
      return i;
    }
  }
  return -1;
}
